'use strict';
Object.defineProperty(exports, '__esModule', {value: true});
exports.default = void 0;

var _Deferred = require('./Deferred');
/** Copyright (c) Facebook, Inc. and its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *
 */ class RSocketTckRequestStreamSubscriber {
  constructor(log) {
    this._cancelled = false;
    this._completeDefer = new _Deferred.Deferred();
    this._completed = false;
    this._errorDefer = new _Deferred.Deferred();
    this._errored = false;
    this._log = log;
    this._payloadCount = null;
    this._payloadDefer = null;
    this._payloads = [];
    this._subscription = null;
  }

  awaitN(n) {
    this._payloadCount = n;
    this._payloadDefer = new _Deferred.Deferred();
    return this._payloadDefer.getPromise();
  }

  awaitTerminal() {
    return Promise.race([
      this._completeDefer.getPromise(),
      this._errorDefer.getPromise(),
    ]);
  }

  getPayloads() {
    return this._payloads;
  }

  isCanceled() {
    return this._cancelled;
  }

  isCompleted() {
    return this._completed;
  }

  hasError() {
    return this._errored;
  }

  isSubscribed() {
    return this._subscription != null;
  }

  cancel() {
    this._cancelled = true;
    if (!this._subscription) {
      throw new Error('subscription is null or undefined');
    }
    this._subscription.cancel();
  }

  request(n) {
    if (!this._subscription) {
      throw new Error('subscription is null or undefined');
    }
    this._subscription.request(n);
  }

  onComplete() {
    this._log('onComplete');
    this._completed = true;
    this._completeDefer.resolve();
    this._errorDefer.reject(new Error('onComplete was called unexpectedly.'));
  }

  onError(error) {
    this._log('onError: %s', error.message);
    this._errored = true;
    this._errorDefer.resolve();
    this._completeDefer.reject(new Error('onError was called unexpectedly.'));
  }

  onNext(payload) {
    this._log('onNext: %s', JSON.stringify(payload));
    this._payloads.push(payload);
    if (this._payloadCount != null && this._payloadDefer != null) {
      this._payloadCount--;
      if (this._payloadCount === 0) {
        this._payloadDefer.resolve();
      }
    }
  }

  onSubscribe(subscription) {
    this._log('onSubscribe');
    this._subscription = subscription;
  }
}
exports.default = RSocketTckRequestStreamSubscriber;
